# Copyright 2011 Element 34
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
===============
SeleniumWrapper
===============
"""
import re
from SaunterSelenium import SaunterSelenium
import time
from selenium import webdriver

class SeleniumWrapper(object):
    """
    Singleton reference to the config information
    """
    _instance = None
    
    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(SeleniumWrapper, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def remote_control(self, host, port, browser, server):
        import tailored.remotecontrol
        self.connection = tailored.remotecontrol.RemoteControl(host, port, browser, server)
        self.connection.running = True
        return self.connection